searchData={"items":[{"type":"behaviour","title":"Splode","doc":"Use this module to create your error aggregator and handler.\n\nFor example:\n\n```elixir\ndefmodule MyApp.Errors do\n  use Splode, error_classes: [\n    invalid: MyApp.Errors.Invalid,\n    unknown: MyApp.Errors.Unknown\n  ],\n  unknown_error: MyApp.Errors.Unknown.Unknown\nend\n```","ref":"Splode.html"},{"type":"callback","title":"Splode.from_json/2","doc":"Converts a combination of a module and json input into an Splode exception.\n\nThis allows for errors to be serialized and deserialized","ref":"Splode.html#c:from_json/2"},{"type":"callback","title":"Splode.set_path/2","doc":"Sets the path on the error or errors","ref":"Splode.html#c:set_path/2"},{"type":"callback","title":"Splode.splode_error?/1","doc":"Returns true if the given value is a splode error.","ref":"Splode.html#c:splode_error?/1"},{"type":"callback","title":"Splode.to_class/1","doc":"Combine errors into an error class","ref":"Splode.html#c:to_class/1"},{"type":"callback","title":"Splode.to_error/1","doc":"Turns any value into a splode error","ref":"Splode.html#c:to_error/1"},{"type":"behaviour","title":"Splode.Error","doc":"Use this module to create an aggregatable error.\n\nFor example:\n\n```elixir\ndefmodule MyApp.Errors.InvalidArgument do\n  use Splode.Error, fields: [:name, :message], class: :invalid\n\n  def message(%{name: name, message: message}) do\n    \"Invalid argument #{name}: #{message}\"\n  end\nend\n```","ref":"Splode.Error.html"},{"type":"callback","title":"Splode.Error.error_class?/0","doc":"","ref":"Splode.Error.html#c:error_class?/0"},{"type":"callback","title":"Splode.Error.from_json/1","doc":"","ref":"Splode.Error.html#c:from_json/1"},{"type":"callback","title":"Splode.Error.splode_error?/0","doc":"","ref":"Splode.Error.html#c:splode_error?/0"},{"type":"type","title":"Splode.Error.t/0","doc":"","ref":"Splode.Error.html#t:t/0"},{"type":"module","title":"Splode.ErrorClass","doc":"Tools for working with error classes","ref":"Splode.ErrorClass.html"},{"type":"function","title":"Splode.ErrorClass.error_messages/2","doc":"Creates a long form composite error message for a list of errors","ref":"Splode.ErrorClass.html#error_messages/2"},{"type":"module","title":"Splode.Stacktrace","doc":"A placeholder for a stacktrace so that we can avoid printing it everywhere","ref":"Splode.Stacktrace.html"},{"type":"type","title":"Splode.Stacktrace.t/0","doc":"","ref":"Splode.Stacktrace.html#t:t/0"},{"type":"exception","title":"Splode.Error.Unknown","doc":"The default top level unknown error container","ref":"Splode.Error.Unknown.html"},{"type":"function","title":"Splode.Error.Unknown.exception/0","doc":"","ref":"Splode.Error.Unknown.html#exception/0"},{"type":"function","title":"Splode.Error.Unknown.message/1","doc":"","ref":"Splode.Error.Unknown.html#message/1"},{"type":"extras","title":"Get Started with Splode","doc":"# Get Started with Splode\n\nSplode helps you deal with errors and exceptions in your application that are aggregatable and consistent. The general pattern is that you use the `Splode` module as a top level aggregator of error classes, and whenever you return errors, you return one of your `Splode.Error` structs, or a string, or a keyword list. Then, if you want to group errors together, you can use your `Splode` module to do so. You can also use that module to turn any arbitrary value into a splode error.\n\nMore documentation for `Splode` will come in the future. This was extracted from Ash Framework so that it could be standardized across multiple packages. If you use Ash, you can use `Ash.Errors` to get the benefits of `Splode`.\n\nFor now, here is an example:\n\n```elixir\ndefmodule MyApp.Errors do\n  use Splode, error_classes: [\n    invalid: MyApp.Errors.Invalid,\n    unknown: MyApp.Errors.Unknown\n  ],\n  unknown_error: MyApp.Errors.Unknown.Unknown\nend\n\n# Error classes are splode errors with an `errors` key.\ndefmodule MyApp.Errors.Invalid do\n  use Splode.ErrorClass, class: :invalid\nend\n\n# You will want to define an unknown error class,\n# otherwise splode will use its own\ndefmodule MyApp.Errors.Unknown do\n  use Splode.ErrorClass, class: :unknown\nend\n\n# This fallback exception will be used for unknown errors\ndefmodule MyApp.Errors.Unknown.Unknown do\n  use Splode.Error, class: :unknown\n\n  # your unknown message should have an `error` key\n  def message(%{error: error}) do\n    if is_binary(error) do\n      to_string(error)\n    else\n      inspect(error)\n    end\n  end\nend\n\n# Finally, you can create your own error classes\n\ndefmodule MyApp.Errors.InvalidArgument do\n  use Splode.Error, fields: [:name, :message], class: :invalid\n\n  def message(%{name: name, message: message}) do\n    \"Invalid argument #{name}: #{message}\"\n  end\nend\n```\n\nTo use these exceptions in your application, the general pattern is to return errors in `:ok | :error` tuples, like so:\n\n```elixir\ndef do_something(argument) do\n  if is_valid?(argument) do\n    {:ok, do_stuff()}\n  else\n    {:error,\n      MyApp.Errors.InvalidArgument.exception(\n        name: :argument,\n        message: \"is invalid\"\n      )}\n  end\nend\n```\n\nThen, you can use `to_class`, and `to_error` tools to ensure that you have consistent error structures.\n\n```elixir\ndef do_multiple_things(argument) do\n  results = [do(), multiple(), things()]\n  {results, errors} =\n    Enum.reduce(results, {[], []}, fn\n      {:ok, result}, {results, errors} ->\n        {[result | results], errors}\n      {:error, error} ->\n        # ensure each error is a splode error\n        # technically, `to_class` does this for you,\n        # this is just an example\n        {results, [MyApp.Errors.to_error(error) | errors]}\n    end)\n\n  case {results, errors} do\n    {results, []} ->\n      {:ok, results}\n    {_results, errors} ->\n      {:error, MyApp.Errors.to_class(errors)}\n  end\nend\n```","ref":"get-started-with-splode.html"},{"type":"extras","title":"Error classes - Get Started with Splode","doc":"When we combine errors into error classes, we choose the first error class for which there are any errors as the \"class\" of the combined error. For example, in Ash Framework, we have:\n\n```elixir\nuse Splode,\n  error_classes: [\n    forbidden: Ash.Error.Forbidden,\n    invalid: Ash.Error.Invalid,\n    framework: Ash.Error.Framework,\n    unknown: Ash.Error.Unknown\n  ],\n  unknown_error: Ash.Error.Unknown.UnknownError\n```\n\nWhat this means is that if there are any `Forbidden` errors, then the class is `Forbidden`. A `Forbidden` error _can_ contain any of the lower classed errors. This allows people to match on and/or rescue on \"the general type of failure\" that occurred. Given that you have many varied kinds of errors, you can use this to your advantage to have both detailed errors, but simple to match on errors. Here is an example:\n\n```elixir\ndef get(conn, %{\"user_id\" => user_id}) do\n  user = MyApp.Accounts.get_user!()\n  render_user(conn, user)\nrescue\n  e in Ash.Error.Forbidden ->\n    render_error(conn, %{error: \"You can't do this\"})\n\n  e in Ash.Error.Invalid ->\n    render_error(conn, %{error: \"You did something wrong\"})\n\n  e in [Ash.Error.Framework, Ash.Error.Unknown] ->\n    render_error(conn, %{error: \"Something went wrong\"})\nend\n```\n\nOr, alternatively, you can pattern match on them given a non-raised error class\n\n```elixir\ndef get(conn, %{\"user_id\" => user_id}) do\n  case MyApp.Accounts.get_user() do\n    {:ok, user} ->\n      render_user(conn, user)\n\n    {:error, %Ash.Error.Forbidden{}} ->\n      render_error(conn, %{error: \"You can't do this\"})\n\n    {:error, %Ash.Error.Invalid{}} ->\n      render_error(conn, %{error: \"You did something wrong\"})\n\n    {:error, %error{}} when error in [Ash.Error.Framework, Ash.Error.Unknown] ->\n      render_error(conn, %{error: \"Something went wrong\"})\n  end\nend\n```\n\n# Raising Exceptions\n\nTo make a `!` version of a function, use `.unwrap!/2` on your splode module.\n\n```elixir\ndef get_user!(user_id) do\n  user_id\n  |> get_user()\n  |> MyApp.Errors.unwrap!()\nend\n\ndef get_user(user_id) do\n  case Repo.get(user_id) do\n    nil ->\n      {:error, MyApp.Error.NotFound.exception(resource: User, key: user_id)}\n    user ->\n      {:ok, user}\n  end\nend\n```","ref":"get-started-with-splode.html#error-classes"},{"type":"extras","title":"Installation - Get Started with Splode","doc":"```elixir\ndef deps do\n  [\n    {:splode, \"~> 0.1.0\"}\n  ]\nend\n```","ref":"get-started-with-splode.html#installation"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.36.1"}}